%  Copyright 2010 Carlos Gonzalez-Aguado
%
%  This file is part of the SAMPLE CODE for the following paper:
%  Bruche, M., Gonzalez-Aguado, C. "Recovery Rates, Default Probabilities, and the Credit Cycle"
%  Journal of Banking and Finance 34(4), 2010, 713-723.
%  
%  SAMPLE CODE is free software: you can redistribute it and/or modify it under
%  the terms of the GNU General Public License as published by the Free
%  Software Foundation, either version 3 of the License, or (at your option)
%  any later version.
%  
%  SAMPLE CODE is distributed in the hope that it will be useful, but WITHOUT
%  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
%  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
%  more details.
%  
%  You should have received a copy of the GNU General Public License along with
%  SAMPLE CODE.  If not, see <http://www.gnu.org/licenses/>.


clear all

% This routine is used to generate data in order to replace the real
% data in our model.

% Parameters
p = 0.9;
q = 0.8;
inistate = 0;
periods = 100;
recperyear = 100;
multiple_fraction = 0.1;

% States of the economy
state_cycle = zeros(periods,1);
state_cycle(1) = inistate;
for i = 1:periods-1
    if state_cycle(i)
        if rand < p
            state_cycle(i+1) = 1;
        end
    else
        if rand > q
            state_cycle(i+1) = 1;
        end
    end
end     

% Macro variable
macro = 0.2 + 0.84*state_cycle + normrnd(0,1,[periods,1]);
save 'macro_data.mat' macro

% Recovery rates (50 recoveries per year)
genparam_recoveries;
j = 1;
for i = 1:periods
    for h = 1:50
        data_matrix(j,1:2) = j;
        data_matrix(j,3) = 0;
        data_matrix(j,4) = int8(rand * 5);
        data_matrix(j,6) = 1980 + i;
        data_matrix(j,7) = 0;
        if (i==1 & h ==1) 
            data_matrix(j,8) = 0;
        else
            if (data_matrix(j,4) > 3)
                data_matrix(j,8) = 0;
            else
                if (rand < 0.1)
                    data_matrix(j,8) = 1;
                end
            end
        end
        
        part1a = a0 + a1*(data_matrix(j,4)==2) + a2*(data_matrix(j,4)==3) + a3*(data_matrix(j,4)==4) + a4*(data_matrix(j,4)==5) + a5*(data_matrix(j,8) == 1)*(1 + a6*(data_matrix(j,4)==2)/a5 + a7*(data_matrix(j,4)==3)/a5);
        part2a = a8 + a9*(data_matrix(j,4)==2) + a10*(data_matrix(j,4)==3) + a11*(data_matrix(j,4)==4) + a12*(data_matrix(j,4)==5) + a13*(data_matrix(j,8) == 1)*(1 + a14*(data_matrix(j,4)==2)/a13 + a15*(data_matrix(j,4)==3)/a13);
        part3a = a16*macro(i) + a17*macro(i)*(data_matrix(j,8)==1);
        part1b = b0 + b1*(data_matrix(j,4)==2) + b2*(data_matrix(j,4)==3) + b3*(data_matrix(j,4)==4) + b4*(data_matrix(j,4)==5) + b5*(data_matrix(j,8) == 1)*(1 + b6*(data_matrix(j,4)==2)/b5 + b7*(data_matrix(j,4)==3)/b5);
        part2b = b8 + b9*(data_matrix(j,4)==2) + b10*(data_matrix(j,4)==3) + b11*(data_matrix(j,4)==4) + b12*(data_matrix(j,4)==5) + b13*(data_matrix(j,8) == 1)*(1 + b14*(data_matrix(j,4)==2)/b13 + b15*(data_matrix(j,4)==3)/b13);
        part3b = b16*macro(i) + b17*macro(i)*(data_matrix(j,8)==1);
        
        a_distr = exp(part1a + part2a*state_cycle(i) + part3a);
        b_distr = exp(part1b + part2b*state_cycle(i) + part3b);

        data_matrix(j,5) = betarnd(a_distr,b_distr);
        j = j + 1;
    end
end
data = data_matrix;
save 'rr_data.mat' data

% Default rates
lambda0 = 3;
lambda1 = 0.5;
lambda2 = 0;
for i = 1:periods
    defrates(i) = 1 / (1 + exp(lambda0 + lambda1*state_cycle(i) + lambda2*macro(i)));
end
defrates = defrates';
save 'defrate.mat' defrates
        








