%  Copyright 2010 Carlos Gonzalez-Aguado
%
%  This file is part of the SAMPLE CODE for the following paper:
%  Bruche, M., Gonzalez-Aguado, C. "Recovery Rates, Default Probabilities, and the Credit Cycle"
%  Journal of Banking and Finance 34(4), 2010, 713-723.
%  
%  SAMPLE CODE is free software: you can redistribute it and/or modify it under
%  the terms of the GNU General Public License as published by the Free
%  Software Foundation, either version 3 of the License, or (at your option)
%  any later version.
%  
%  SAMPLE CODE is distributed in the hope that it will be useful, but WITHOUT
%  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
%  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
%  more details.
%  
%  You should have received a copy of the GNU General Public License along with
%  SAMPLE CODE.  If not, see <http://www.gnu.org/licenses/>.


% This function calculates the likelihood function
% The function 'pre_lik' calculates the likelihood for each possible combination of latent variables (in recoveries, defaults, both, or none)

function f = lik(x)

% Declare global variables
global dr_cycle rr_cycle dr_mat dr_mat0 dr_mat1 rr_mat0 rr_mat1 rr_mat pars t fil upd

% Obtain the likelihoof function
if dr_cycle | rr_cycle
    if dr_cycle & ~rr_cycle
        f0 = pre_lik(x,dr_mat0,rr_mat);
        f1 = pre_lik(x,dr_mat1,rr_mat);
    else
        if rr_cycle & ~dr_cycle
            f0 = pre_lik(x,dr_mat,rr_mat0);
            f1 = pre_lik(x,dr_mat,rr_mat1);
        else
            f0 = pre_lik(x,dr_mat0,rr_mat0);
            f1 = pre_lik(x,dr_mat1,rr_mat1);
        end
    end
    % Probabilities
    p = 1 / (1 + exp(x(pars(4,1))));
    q = 1 / (1 + exp(x(pars(4,1)+1)));
    init = 1 / (1 + exp(x(pars(4,1)+2)));
    % Hamilton filter (upd - updated probs; fil - filtered probs)	
    upd = zeros(t,1);
    fil = zeros(t,1);
    f = zeros(t,1);
    upd(1) = init;
    for i = 2:t
        upd(i) = (1-q) + (p + q -1)*f1(i-1)*upd(i-1)/(f0(i-1) + upd(i-1)*(f1(i-1) - f0(i-1)));
    end
    g = upd.*f1 + (1-upd).*f0;
    fil = (f1 .* upd) ./ g;    
else
    g = pre_lik(x,dr_mat,rr_mat);
end

% We calculate the minus-loglikelihood function (in order to minimize it)
f = -sum(log(g))
